//    LightMP3
//    Copyright (C) 2007 Sakya
//    sakya_tg@yahoo.it
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#include <pspkernel.h>
#include <pspctrl.h>
#include <psphprm.h> 
#include <pspdebug.h>
#include <pspaudio.h>
#include <pspaudiolib.h>
#include <psppower.h>

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <pspsdk.h>
#include <unistd.h>

#include "mp3player.h"
#include "opendir.h"
#include "id3.h"
#include "m3u.h"
#include "equalizer.h"

//#include "log.h"

//PSP_MODULE_INFO("LightMP3", 0, 1, 1);
PSP_MODULE_INFO("LightMP3", 0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0);


void sceDisplay_driver_9E3C6DC6(int a0,int a1);//a0 0-100,a1 = 0/1 (set to 0)
#define sceDisplaySetBrightness sceDisplay_driver_9E3C6DC6
void sceDisplay_driver_31C4BAA8(int *a0,int *a1);
#define sceDisplayGetBrightness sceDisplay_driver_31C4BAA8
int sceDisplayEnable(void);
int sceDisplayDisable(void); 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Constants:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Colori:
#define RGB(r, g, b) ((b << 16) | (g << 8) | r)
#define YELLOW RGB(255, 255, 0)
#define ORANGE RGB(255, 165, 0)
#define BLACK RGB(0, 0, 0)
#define WHITE RGB(255, 255, 255)
#define RED RGB(255, 0, 0)
#define DEEPSKYBLUE RGB(0, 178, 238)

//Altre costanti
#define mp3_directory_1 "ms0:/MUSIC"
#define mp3_directory_2 "ms0:/PSP/MUSIC"
#define MUTED_VOLUME 0x800
//#define MUTED_VOLUME 0
#define MAX_VOLUME_BOOST 4

//Costanti per playing mode:
#define MODE_NORMAL 0
#define MODE_REPEAT 1
#define MODE_SHUFFLE 2

//Sensibilit analogico:
#define ANALOG_SENS 50

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int current_mode;
char version[10] = "1.2.0";
int oldBrightness = 0;
char ebootDirectory[257] = "";
char currentPlaylist[257] = "";
int playListModified;
char tempM3Ufile[257] = "";
char fileBrowserDir[257] = "";
char playlistDir[257] = "";
int playingMode = 0;
char playingModeDesc[3][10] = {"Normal ", "Repeat ", "Shuffle"};
int volumeBoost = 0;

int curBrightness = 0;
int displayStatus = 1;
int currentEQ = 0;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Callbacks:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
          sceKernelExitGame();
          return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp) {
          int cbid;

          cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
          sceKernelRegisterExitCallback(cbid);

          sceKernelSleepThreadCB();

          return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void) {
          int thid = 0;

          thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
          if(thid >= 0) {
                    sceKernelStartThread(thid, 0, 0);
          }

          return thid;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funzioni generiche
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Memoria libera totale:
typedef struct
{
      void *buffer;
      void *next;
} _LINK;

int freemem()
{
   int size = 4096, total = 0;
    _LINK *first = NULL, *current = NULL, *lnk;

    while (1){
       lnk = (_LINK*)malloc(sizeof(_LINK));
       if (!lnk)
          break;

       total += sizeof(_LINK);

       lnk->buffer = malloc(size);
       if (!lnk->buffer){
          free(lnk);
          break;
       }

       total += size;
       lnk->next = NULL;

       if (current){
          current->next = (void*)lnk;
          current = lnk;
       } else {
          current = first = lnk;
       }
    }

    lnk = first;
    while (lnk){
       free(lnk->buffer);
       current = lnk->next;
       free(lnk);
       lnk = current;
    }
    return total;
}

//Modalit precedente/successiva:
void nextMode(){
	if (++current_mode > 2){
		current_mode = 0;
	}
}
void previousMode(){
	if (--current_mode < 0){
		current_mode = 2;
	}
}

//Ordinamento dei file di una directory:
void sortDirectory(struct opendir_struct directory)
{
	int n = directory.number_of_directory_entries;
	int i = 0;
	
	while (i < n) {
		if (i == 0 || strcmp(directory.directory_entry[i-1].d_name, directory.directory_entry[i].d_name) <= 0) i++;
		else {SceIoDirent tmp = directory.directory_entry[i]; directory.directory_entry[i] = directory.directory_entry[i-1]; directory.directory_entry[--i] = tmp;}
	}
}

//Prendo solo il nome del file/directory:
void getFileName(char *fileName, char *onlyName){
	int slash = -1;
	int retCount;

	strcpy(onlyName, fileName);
	//Cerco l'ultimo slash:
	int i = 0;
	for (i = strlen(fileName) - 1; i >= 0; i--){
		if ((fileName[i] == '/') & (i != strlen(fileName))){
			slash = i;
			break;
		}
	}
	if (slash){
		retCount = 0;
		for (i = slash + 1; i < strlen(fileName); i++){
			onlyName[retCount] = fileName[i];
			retCount++;
		}
		onlyName[retCount] = '\0';
	}else{
		strcpy(onlyName, fileName);
	}
}

//Calcolo livello superiore:
int directoryUp(char *dirName)
{
	if (dirName != "ms0:/"){
		//Cerco l'ultimo slash:
		int i = 0;
		for (i = strlen(dirName) - 1; i >= 0; i--){
			if ((dirName[i] == '/') & (i != strlen(dirName))){
				if (i > 4){
					dirName[i] = '\0';
				}else{
					dirName[i+1] = '\0';
				}
				break;
			}
		}
		return(0);
	}else{
		return(-1);
	}
}

//Spezzo autore e titolo nel nome di una directory:
void rtrim(char *toTrim)
{
	int i = 0;
	for (i = strlen(toTrim) - 1; i >= 0; i--){
		if (isspace(toTrim[i])){
			toTrim[i] = '\0';
		}else{
			break;
		}
	}
}

int splitAuthorTitle(char *dirName, char *author, char *title)
{
	if (strstr(dirName, "-") != NULL){
		int i = 0;
		int posSep = -1;
		int countAuthor = 0;
		int countTitle = 0;

		for (i = 0; i < strlen(dirName); i++){
			if (dirName[i] == '-'){
				posSep = i;
			}
			else if (posSep == -1){
				if ((countAuthor) | (dirName[i] != ' ')){
					author[countAuthor] = dirName[i];
					countAuthor++;
				}
			}else{
				if ((countTitle) | (dirName[i] != ' ')){
					title[countTitle] = dirName[i];
					countTitle++;
				}
			}
			author[countAuthor] = '\0';
			title[countTitle] = '\0';
		}
		rtrim(author);
		rtrim(title);
		return(1);
	}
	return(0);
}

//Costruisco la barra della percentuale:
void buildProgressBar(char *pBar, int percentage){
	int i;

	pBar[0] = '[';
	for (i = 1; i < 22; i++){
		if (percentage >= (i-1) * 5){
			pBar[i] = '*';
		}else{
			pBar[i] = ' ';
		}
	}
	pBar[21] = ']';
	pBar[22] = '\0';
}

//Chiedo conferma di una azione:
int confirm(char *message){
	int i;
	pspDebugScreenSetBackColor(BLACK);

	for (i = 0; i < 5; i++){
		pspDebugScreenSetXY(15, 10+i);
		pspDebugScreenPrintf("%-38.38s", "");
		if (i == 0){
			pspDebugScreenSetTextColor(RED);
			pspDebugScreenSetXY(15 + (38 - strlen("Confirm")) / 2, 10+i);
			pspDebugScreenPrintf("%s", "Confirm");
		}else if (i == 2){
			pspDebugScreenSetTextColor(WHITE);
			pspDebugScreenSetXY(15 + (38 - strlen(message)) / 2, 10+i);
			pspDebugScreenPrintf("%s", message);
		}else if (i == 4){
			pspDebugScreenSetTextColor(YELLOW);
			pspDebugScreenSetXY(15, 10+i);
			pspDebugScreenPrintf(" Press X to confirm, CIRCLE to cancel ");
		}
	}
	sceKernelDelayThread(200000);
	SceCtrlData pad;
	while (1){
		sceCtrlReadBufferPositive(&pad, 1);
		if (pad.Buttons & PSP_CTRL_CROSS){
			sceKernelDelayThread(200000);
			return(1);
		} else if (pad.Buttons & PSP_CTRL_CIRCLE) {
			sceKernelDelayThread(200000);
			return(0);
		}
	}
}


//Draw a waiting message:
void waitMessage(char *message){
	int i;
	int max = 0;
	if (strlen(message) == 0){
		max = 3;
	}else{
		max = 4;
	}
	pspDebugScreenSetBackColor(BLACK);
	for (i=0; i < max; i++){
		pspDebugScreenSetXY(19, 10+i);
		pspDebugScreenPrintf("%-30.30s", "");
		if (i == 1){
			pspDebugScreenSetTextColor(YELLOW);
			pspDebugScreenSetXY(19 + (30 - strlen("Please wait...")) / 2, 10+i);
			pspDebugScreenPrintf("%s", "Please wait...");
		}else if (i == 2 && strlen(message) > 0){
			pspDebugScreenSetTextColor(WHITE);
			pspDebugScreenSetXY(19 + (30 - strlen(message)) / 2, 10+i);
			pspDebugScreenPrintf("%s", message);
		}
	}
}

//Insert a space in a string:
int insertSpace(char *string, int pos){
	int i;
	int count = 0;
	char tempStr[strlen(string)];

	for (i=0; i<strlen(string); i++){
		if (i == pos){
			tempStr[count] = ' ';
			count++;
		}
		tempStr[count] = string[i];
		count++;
	}
	tempStr[count] = '\0';
	strcpy(string, tempStr);
	return(0);
}

//Remove a char from a string:
int removeChar(char *string, int pos){
	int i;
	int count = 0;
	char tempStr[strlen(string)];

	for (i=0; i<strlen(string); i++){
		if (i == pos){
			continue;
		}
		tempStr[count] = string[i];
		count++;
	}
	tempStr[count] = '\0';
	strcpy(string, tempStr);
	return(0);
}

//Ask for a string (for playlist name)
int askString(char *string){
	int i;
	int cursorPos = 0;
	int changed = 0;
	char character[2] = "";
	char oldName[257] = "";
	int asciiCode = (int)' ';

	//Salvo il nome:
	strcpy(oldName, string);

	pspDebugScreenSetBackColor(BLACK);
	for (i=0; i < 5 ; i++){
		pspDebugScreenSetXY(11, 10+i);
		pspDebugScreenPrintf("%-46.46s", "");		
	}
	pspDebugScreenSetTextColor(RED);
	pspDebugScreenSetXY(11 + (46 - strlen("Insert playlist name")) / 2, 10);
	pspDebugScreenPrintf("%s", "Insert playlist name");
	pspDebugScreenSetTextColor(YELLOW);
	pspDebugScreenSetXY(11, 13);
	pspDebugScreenPrintf("Press X to insert space, SQUARE to delete char");
	pspDebugScreenSetXY(11, 14);
	pspDebugScreenPrintf("   Press START to confirm, SELECT to cancel");
	sceKernelDelayThread(200000);
	SceCtrlData pad;
	//Primo carattere:
	if (strlen(string) == 0){
		strcpy(string, " ");
		asciiCode = (int)' ';
	}else{
		asciiCode = (int)string[0];
	}
	changed = 1;

	while (1){
		if (changed == 1){
			changed = 0;
			//String:
			pspDebugScreenSetXY(11, 12);
			pspDebugScreenSetTextColor(WHITE);
			pspDebugScreenSetBackColor(BLACK);
			pspDebugScreenPrintf("%-46.46s", string);
			//Cursor:
			pspDebugScreenSetBackColor(WHITE);
			pspDebugScreenSetTextColor(BLACK);
			pspDebugScreenSetXY(11 + cursorPos, 12);
			character[0] = (char)asciiCode;
			pspDebugScreenPrintf(character);
		}
		sceCtrlReadBufferPositive(&pad, 1);
		if (pad.Buttons & PSP_CTRL_START){
			//Conferma nome:
			rtrim(string);
			sceKernelDelayThread(200000);
			if (strlen(string) == 0){
				return(-1);
			}
			break;
		}else if (pad.Buttons & PSP_CTRL_SELECT){
			//Restore del nome:
			strcpy(string, oldName);
			sceKernelDelayThread(200000);
			return(-1);
		}else if (pad.Buttons & PSP_CTRL_CROSS){
			//Inserisco uno spazio:
			if (strlen(string) <= 45){
				insertSpace(string, cursorPos);
				asciiCode = (int)' ';
				changed = 1;
				sceKernelDelayThread(200000);
			}
		}else if (pad.Buttons & PSP_CTRL_SQUARE){
			//Cancello un carattere:
			removeChar(string, cursorPos);
			if (cursorPos > strlen(string) - 1){
				if (strlen(string) > 0){
					cursorPos = strlen(string) - 1;
					asciiCode = (int)string[cursorPos];
				}else{
					cursorPos = 0;
					strcpy(string, " ");
					asciiCode = (int)' ';
				}
			}else{
				asciiCode = (int)string[cursorPos];
			}
			changed = 1;
			sceKernelDelayThread(200000);
		}else if ((pad.Buttons & PSP_CTRL_RIGHT) | (pad.Lx > 128 + ANALOG_SENS)){
			if (cursorPos < 45){
				cursorPos++;
				if (cursorPos < strlen(string) - 1){
					asciiCode = (int)string[cursorPos];
				}else{
					asciiCode = (int)' ';
					strcat(string, " ");
				}
				changed = 1;
				sceKernelDelayThread(200000);
			}
		}else if ((pad.Buttons & PSP_CTRL_LEFT) | (pad.Lx < 128 - ANALOG_SENS)){
			if (cursorPos > 0){
				cursorPos--;
				if (cursorPos < strlen(string) - 1){
					asciiCode = (int)string[cursorPos];
				}else{
					asciiCode = (int)' ';
				}
				changed = 1;
			}
			sceKernelDelayThread(200000);
		} else if((pad.Buttons & PSP_CTRL_UP) | (pad.Ly < 128 - ANALOG_SENS)) {
			if (asciiCode < (int)' ' + 94){
				asciiCode++;
				string[cursorPos] = (char)asciiCode;
			}else{
				asciiCode = (int)' ';
			}
			changed = 1;
			sceKernelDelayThread(120000);
		} else if((pad.Buttons & PSP_CTRL_DOWN) | (pad.Ly > 128 + ANALOG_SENS)) {
			if (asciiCode > (int)' '){
				asciiCode--;
				string[cursorPos] = (char)asciiCode;
			}else{
				asciiCode = (int)' ' + 94;
			}
			changed = 1;
			sceKernelDelayThread(120000);
		}
	}
	return(0);
}

//Random track for shuffle:
int randomTrack(int max, char *used){
	int random;
	char chrRand[2] = "";
	if (strlen(used) == max){
		return(-1);
	}
	random = rand()%max;
	chrRand[0] = (char)(random+1);
	while (strstr(used, chrRand) != NULL){
		random = rand()%max;
		chrRand[0] = (char)(random+1);
	}
	strcat(used, chrRand);
	return(random);
}

//Aggiungo un file alla playlist temporanea:
void addFileToPlaylist(char *fileName, int save){
	struct ID3Tag ID3;
	struct MP3Info info;
	ID3 = ParseID3(fileName);
	MP3_Init(1);
	MP3_Load(fileName);
	info = MP3_GetInfo();
	MP3_End();
	M3U_addSong(fileName, mad_timer_count(info.length, MAD_UNITS_SECONDS), ID3.ID3Title);
	if (save == 1){
		M3U_save(tempM3Ufile);
	}
}

//Aggiungo una directory alla playlist temporanea:
void addDirectoryToPlaylist(char *dirName){
	char ext[1][4] = {"MP3"};
	char fileToAdd[257] = "";
	char message[10] = "";
	int i;
	float perc;
	struct opendir_struct dirToAdd;

	char *result = opendir_open(&dirToAdd, dirName, ext, 1, 0);
	if (result == 0){
		for (i = 0; i < dirToAdd.number_of_directory_entries; i++){
			strcpy(fileToAdd, dirName);
			if (fileToAdd[strlen(fileToAdd)-1] != '/'){
				strcat(fileToAdd, "/");
			}
			strcat(fileToAdd, dirToAdd.directory_entry[i].d_name);
			addFileToPlaylist(fileToAdd, 0);
			perc = ((float)(i + 1) / (float)dirToAdd.number_of_directory_entries) * 100.0;
			snprintf(message, sizeof(message), "%3.3i%% done", (int)perc);
			waitMessage(message);
		}
		M3U_save(tempM3Ufile);
	}
	opendir_close(&dirToAdd);
}


//Check brightness and warning:
void checkBrightness(){
	int i;
	sceDisplayGetBrightness(&curBrightness, 0);

	if (curBrightness > 24){
		pspDebugScreenSetTextColor(WHITE);
		pspDebugScreenSetBackColor(0xaa4400);
		pspDebugScreenInit();

		pspDebugScreenSetBackColor(BLACK);
		for (i = 0; i < 5; i++){
			pspDebugScreenSetXY(15, 13+i);
			pspDebugScreenPrintf("%-38.38s", "");
			if (i == 0){
				pspDebugScreenSetTextColor(RED);
				pspDebugScreenSetXY(15 + (38 - strlen("Brightness warning")) / 2, 13+i);
				pspDebugScreenPrintf("%s", "Brightness warning");
			}else if (i == 2){
				pspDebugScreenSetTextColor(WHITE);
				pspDebugScreenSetXY(15 + (38 - strlen("Lower display brightness?")) / 2, 13+i);
				pspDebugScreenPrintf("%s", "Lower display brightness?");
			}else if (i == 4){
				pspDebugScreenSetTextColor(YELLOW);
				pspDebugScreenSetXY(15, 13+i);
				pspDebugScreenPrintf(" Press X to confirm, CIRCLE to cancel ");
			}
		}
		sceKernelDelayThread(200000);
		SceCtrlData pad;
		while (1){
			sceCtrlReadBufferPositive(&pad, 1);
			if (pad.Buttons & PSP_CTRL_CROSS){
				sceDisplaySetBrightness(24, 0);
				curBrightness = 24;
				break;
			} else if (pad.Buttons & PSP_CTRL_CIRCLE) {
				break;
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Inizializzazioni video:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Stringa impostazioni player:
void screen_playerSettings(){
	char mode[2] = "";
	struct equalizer tEQ;

	tEQ = EQ_getIndex(currentEQ);
	if (playingMode == MODE_NORMAL){
		mode[0] = 'N';
	}else if (playingMode == MODE_REPEAT){
		mode[0] = 'R';
	}else if (playingMode == MODE_SHUFFLE){
		mode[0] = 'S';
	}
	pspDebugScreenSetBackColor(0x882200);
	pspDebugScreenSetTextColor(RED);
	pspDebugScreenSetXY(24, 33);
	pspDebugScreenPrintf("Mode:%s Boost:%i EQ:%-2.2s", mode, volumeBoost, tEQ.shortName);
}

//System info:
void screen_sysinfo(){
    int m;
    int h;
    int btrh;
    int btrm;
    int btr; 
    int rh; 
	int mem;
	int battPerc;
	btr = scePowerGetBatteryLifeTime();
    h = 60;
    m = 60;
    rh = ((btr/60)/60);
    btrm = (btr%m);
    btrh = (btr/h + rh); 

	battPerc = scePowerGetBatteryLifePercent();
	if (battPerc < 0){
		battPerc = 0;
	}
	if (btrh < 0 || btrm < 0){
		btrh = 0;
		btrm = 0;
	}
	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetBackColor(0x882200);
	pspDebugScreenSetXY(26, 0);
	if (!scePowerIsPowerOnline()){
		pspDebugScreenPrintf("[CPU:%i BUS:%i BATT:%i%% - %2.2i:%2.2i]", scePowerGetCpuClockFrequency(), scePowerGetBusClockFrequency(), battPerc, btrh, btrm);
	}else{
		pspDebugScreenPrintf("[CPU:%i BUS:%i BATT:%i%% -PLUGGED]", scePowerGetCpuClockFrequency(), scePowerGetBusClockFrequency(), battPerc);
	}

	//Free memory:
	mem = freemem() / 1024;
	pspDebugScreenSetXY(0, 33);
	pspDebugScreenPrintf("Free mem: %i kb", mem);
	//Current time:
	struct tm * ptm;
	time_t mytime;
	time(&mytime);
	ptm = localtime(&mytime);
	pspDebugScreenSetXY(51, 33);
	pspDebugScreenPrintf("%2.2d/%2.2d/%4.4d %2.2d:%2.2d",ptm->tm_mday, ptm->tm_mon + 1, ptm->tm_year + 1900, ptm->tm_hour,ptm->tm_min);
	screen_playerSettings();
}

//Inizializzazione video:
void screen_init()
	{
	char testo[67] = "";
	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetBackColor(BLACK);
	pspDebugScreenInit();
	pspDebugScreenSetBackColor(0x882200);
	//Righe sopra e sotto:
	pspDebugScreenSetXY(0, 0);
	snprintf(testo, sizeof(testo), "LightMP3 v%s by Sakya", version);
	pspDebugScreenPrintf("%-67.67s",testo);
	pspDebugScreenSetXY(0, 33);
	pspDebugScreenPrintf("%-67.67s", "");
	screen_sysinfo();
	//Riga descrizione:
	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetBackColor(BLACK);
	pspDebugScreenSetXY(0, 1);
	pspDebugScreenPrintf("a light MP3 Player to preserve your battery. ;)");
	pspDebugScreenSetXY(0, 33);
}

//Schermo per menu
void screen_menu_init()
{
	pspDebugScreenSetTextColor(RED);
	pspDebugScreenSetXY(0, 2);
	pspDebugScreenPrintf("Mode: File browser");
	pspDebugScreenSetBackColor(BLACK);
	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetXY(0, 27);
	pspDebugScreenPrintf("Press X to enter directory/play file");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press SQUARE to play directory");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press O to go up one level");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press TRIANGLE to exit");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press START to add file/directory to playlist");

}

//Schermo per playlist
void screen_playlist_init()
{
	pspDebugScreenSetTextColor(RED);
	pspDebugScreenSetXY(0, 2);
	pspDebugScreenPrintf("Mode: Playlist browser");
	pspDebugScreenSetBackColor(BLACK);
	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetXY(0, 27);
	pspDebugScreenPrintf("Press X to play playlist");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press SQUARE to load playlist");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press CIRCLE to remove playlist");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press TRIANGLE to exit");
}

//Schermo per playlist editor
void screen_playlistEditor_init()
{
	pspDebugScreenSetTextColor(RED);
	pspDebugScreenSetXY(0, 2);
	pspDebugScreenPrintf("Mode: Playlist editor");
	pspDebugScreenSetBackColor(BLACK);
	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetXY(0, 26);
	pspDebugScreenPrintf("Press X to move track down");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press SQUARE to move track up");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press CIRCLE to remove track from playlist");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press START to save playlist");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press SELECT to clear playlist");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press NOTE to play the playlist");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press TRIANGLE to exit");
}

//Schermo per player
void screen_player_init(char *filename, struct ID3Tag ID3, int nextPrevious, char *numbers)
{
	char file[257] = "";
	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetBackColor(BLACK);
	if (nextPrevious == 1){
	    pspDebugScreenSetXY(0, 7);
		pspDebugScreenPrintf("File Number: %s", numbers);
	}
    pspDebugScreenSetXY(0, 8);
	getFileName(filename, file);
	pspDebugScreenPrintf("File Name  : %-54.54s", file);
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Time       :");
	pspDebugScreenSetTextColor(YELLOW);
    pspDebugScreenSetXY(0, 11);
    pspDebugScreenPrintf("Album      : %s", ID3.ID3Album);
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Title      : %s", ID3.ID3Title);
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Artist     : %s", ID3.ID3Artist);
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Year       : %s", ID3.ID3Year);
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Genre      : %s", ID3.ID3GenreText);

	pspDebugScreenSetTextColor(DEEPSKYBLUE);
	pspDebugScreenSetXY(0, 16);
    pspDebugScreenPrintf("Layer      :");
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Bitrate    :");
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Sample rate:");
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Mode       :");
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Emphasis   :");

	pspDebugScreenSetTextColor(WHITE);
	pspDebugScreenSetXY(0, 25);
	pspDebugScreenPrintf("Press X to pause");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press O to stop");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press SQUARE toggle mute");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press SELECT to switch mode");
	if (nextPrevious == 1)
	{
		pspDebugScreenPrintf("\n");
		pspDebugScreenPrintf("Press L/R to change track");
	}
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press UP/DOWN to change volume boost value");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press START to turn on/off the display");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Use analog to change clock (UP/DOWN for cpu, LEFT/RIGHT for bus)");
}

//Informazioni sul file:
void screen_fileinfo(struct MP3Info info)
{
    pspDebugScreenSetXY(0, 16);
	pspDebugScreenSetTextColor(DEEPSKYBLUE);
    pspDebugScreenPrintf("Layer      : %s", info.layer);
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Bitrate    : %i kbit", info.kbit);
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Sample rate: %li Hz", info.hz);
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Mode       : %s", info.mode);
	pspDebugScreenPrintf("\n");
    pspDebugScreenPrintf("Emphasis   : %s", info.emphasis);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Play a single MP3 file:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int playMP3(char *filename, int nextPrevious, char *numbers, char *message) {
	  /* Play a single file.
		 Returns 1 if user pressed NEXT
				 0 if song ended
				-1 if user pressed PREVIOUS
				 2 if user pressed STOP
	  */
	  u32 remoteButtons; 
	  int retVal = 0;
	  SceCtrlData pad;
	  int clock;
	  int bus;
	  int action = 0; //-1=pause 0=stop 1=playing 
	  char timestring[20] = "";
	  int updateInfo = 0;
	  struct ID3Tag ID3;
	  struct MP3Info info;
	  char pBar[22] = "";
	  int muted = 0;
	  struct equalizer tEQ;

	  screen_init();
	  info.kbit = 0;
	  info.hz = 0;
	
	  clock = scePowerGetCpuClockFrequency();
	  bus = scePowerGetBusClockFrequency();
	  //Leggo le informazioni ID3:
	  ID3 = ParseID3(filename);
	  screen_player_init(filename, ID3, nextPrevious, numbers);
	  //Testo del modo di riproduzione:
	  pspDebugScreenSetXY(0, 4);
	  pspDebugScreenSetBackColor(BLACK);
	  pspDebugScreenSetTextColor(YELLOW);
	  pspDebugScreenPrintf("Play mode  : %s", playingModeDesc[playingMode]);

	  MP3_Init(1);
	  MP3_setVolumeBoost(volumeBoost);
	  pspAudioSetVolume(1, 0x8000, 0x8000);
	  pspDebugScreenSetXY(0, 21);
	  pspDebugScreenSetTextColor(YELLOW);
   	  tEQ = EQ_getIndex(currentEQ);
	  pspDebugScreenPrintf("Equalizer  : %-20.20s", tEQ.name);
	  pspDebugScreenSetXY(0, 22);
	  pspDebugScreenSetTextColor(WHITE);
	  pspDebugScreenPrintf("Opening...");
	  if (strlen(message) > 0){
		  pspDebugScreenSetXY(0, 5);
		  pspDebugScreenPrintf(message);
	  }

	  //Apro il file:
	  MP3_Load(filename);
	  info = MP3_GetInfo();
	  screen_fileinfo(info);
	  MP3_Play();
	  action = 1;

	  while(1){
		    //Aggiorno info a video:
			if (updateInfo == 0){
				screen_sysinfo();
				MP3_GetTimeString(timestring);
				pspDebugScreenSetTextColor(WHITE);
				pspDebugScreenSetBackColor(BLACK);
				pspDebugScreenSetXY(0, 9);
				pspDebugScreenPrintf("Time       : %s / %s", timestring, info.strLength);
				buildProgressBar(pBar, MP3_GetPercentace());
				pspDebugScreenSetXY(13, 10);
				pspDebugScreenPrintf(pBar);
				//Testo dell'azione corrente:
				pspDebugScreenSetXY(0, 22);
				pspDebugScreenSetTextColor(WHITE);
				if (action == 1){
					pspDebugScreenPrintf("Playing...");
				}else if (action == -1){
					pspDebugScreenPrintf("Paused    ");
				}
				//Impedisco che il video vada in standby se l'ho spento io:
				if (displayStatus == 0){
					scePowerTick(0);
				}
			}
			updateInfo++;
			if (updateInfo == 10){
				updateInfo = 0;
			}
			sceCtrlReadBufferPositive(&pad, 1);
			sceHprmPeekCurrentKey(&remoteButtons); 

			if(pad.Buttons & PSP_CTRL_CIRCLE) {
					  MP3_End();
					  action = 0;
					  retVal = 2;
					  break;
			} else if((pad.Buttons & PSP_CTRL_CROSS) | (remoteButtons & PSP_HPRM_PLAYPAUSE)) {
					  MP3_Pause();
					  if (action != -1){
						action = -1;
					  }else{
						action = 1;
					  }
					  updateInfo = 0;
					  sceKernelDelayThread(400000);
			} else if(pad.Buttons & PSP_CTRL_UP){
				//Boost volume up:
				if (volumeBoost < MAX_VOLUME_BOOST){
					MP3_setVolumeBoost(++volumeBoost);
					screen_playerSettings();
				}
				sceKernelDelayThread(400000);
			} else if(pad.Buttons & PSP_CTRL_DOWN){
				//Boost volume down:
				if (volumeBoost > 0){
					MP3_setVolumeBoost(--volumeBoost);
					screen_playerSettings();
				}
				sceKernelDelayThread(400000);
			} else if(pad.Ly > 128 + ANALOG_SENS) {
				if (clock > 33){
					clock--;
					scePowerSetCpuClockFrequency(clock);
					screen_sysinfo();
					sceKernelDelayThread(100000);
				}
			} else if(pad.Ly < 128 - ANALOG_SENS) {
				if (clock < 333){
					clock++;
					scePowerSetCpuClockFrequency(clock);
					screen_sysinfo();
					sceKernelDelayThread(100000);
				}
			} else if(pad.Lx < 128 - ANALOG_SENS) {
				if (bus > 54){
					bus--;
					scePowerSetBusClockFrequency(bus);
					screen_sysinfo();
					sceKernelDelayThread(100000);
				}
			} else if(pad.Lx > 128 + ANALOG_SENS) {
				if (clock < 166){
					bus++;
					scePowerSetBusClockFrequency(bus);
					screen_sysinfo();
					sceKernelDelayThread(100000);
				}
			} else if(pad.Buttons & PSP_CTRL_LTRIGGER || remoteButtons & PSP_HPRM_BACK) {
				if (nextPrevious == 1){
					MP3_End();
					retVal = -1;
					break;
				}
			} else if(pad.Buttons & PSP_CTRL_RTRIGGER || remoteButtons & PSP_HPRM_FORWARD) {
				if (nextPrevious == 1){
					MP3_End();
					retVal = 1;
					break;
				}
			} else if(pad.Buttons & PSP_CTRL_START) {
				if (displayStatus == 1){
					//Spengo il display:
					sceDisplayGetBrightness(&curBrightness, 0);
					sceDisplayDisable();
					displayStatus = 0;
				} else {
					//Accendo il display:	
					sceDisplayEnable();
					//sceDisplaySetBrightness(curBrightness, 0);
					displayStatus = 1;
				}
				sceKernelDelayThread(400000);
			} else if(pad.Buttons & PSP_CTRL_SQUARE) {
				//Mute solo se sono in play:
				if (action == 1){
					pspDebugScreenSetXY(0, 23);
					if (muted == 0){
						pspAudioSetVolume(1, MUTED_VOLUME, MUTED_VOLUME);
						muted = 1;
						pspDebugScreenSetTextColor(RED);
						pspDebugScreenPrintf("Mute ON");
					}else{
						pspAudioSetVolume(1, 0x8000, 0x8000);
						muted = 0;
						pspDebugScreenSetBackColor(BLACK);
						pspDebugScreenPrintf("       ");
					}
					updateInfo = 0;
					sceKernelDelayThread(400000);
				}
			} else if(pad.Buttons & PSP_CTRL_SELECT) {
				//Change playing mode:
				if (++playingMode > MODE_SHUFFLE){
					playingMode = MODE_NORMAL;
				}
				pspDebugScreenSetXY(0, 4);
				pspDebugScreenSetBackColor(BLACK);
				pspDebugScreenSetTextColor(YELLOW);
				pspDebugScreenPrintf("Play mode  : %s", playingModeDesc[playingMode]);
				screen_playerSettings();
				sceKernelDelayThread(400000);
			} else if(pad.Buttons & PSP_CTRL_NOTE){
				//Cambio equalizzatore:
				if (++currentEQ >= EQ_getEqualizersNumber()){
					currentEQ = 0;
					MP3_disableFilter();
				}else{
					MP3_enableFilter();
				}
				tEQ = EQ_getIndex(currentEQ);
				MP3_setFilter(tEQ.filter);
				pspDebugScreenSetXY(0, 21);
				pspDebugScreenSetBackColor(BLACK);
				pspDebugScreenSetTextColor(YELLOW);
				pspDebugScreenPrintf("Equalizer  : %-20.20s", tEQ.name);
				screen_playerSettings();
				sceKernelDelayThread(400000);
			}

			//Controllo se l'mp3  finito:
			if (MP3_EndOfStream() == 1) {
				//Controllo il repeat:
				if (playingMode == MODE_REPEAT && nextPrevious == 0){
					MP3_Play();
				}else{
					MP3_End();
					retVal = 0;
					break;
				}
			}
	  }
	  //Accendo il display se devo:	
	  if (displayStatus == 0 && retVal == 2){
	    sceDisplayEnable();
	    //sceDisplaySetBrightness(curBrightness, 0);
	  }
	  return(retVal);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Play directory:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void playDirectory(char *dirName){	
	struct opendir_struct dirToPlay;
	char ext[1][4] = {"MP3"};
	char testo[10] = "";
	char fileToPlay[257] = "";
	char message[60] = "";

	char *result = opendir_open(&dirToPlay, dirName, ext, 1, 0);
	char playedTracks[dirToPlay.number_of_directory_entries + 1];
	if (result == 0){
		int i = 0;
		int playerReturn = 0;
		char onlyName[257] = "";

		sortDirectory(dirToPlay);
		getFileName(dirName, onlyName);
		snprintf(message, sizeof(message), "Directory  : %-60.60s", onlyName);

		playedTracks[0] = '\0';
		if (playingMode == MODE_SHUFFLE){
			i = randomTrack(dirToPlay.number_of_directory_entries, playedTracks);
		}else{
			i = 0;
		}

		while (1){
			snprintf(testo, sizeof(testo), "%i/%i", i + 1, dirToPlay.number_of_directory_entries);
			strcpy(fileToPlay, dirName);
			if (dirName[strlen(dirName)-1] != '/'){
				strcat(fileToPlay, "/");
			}
			strcat(fileToPlay, dirToPlay.directory_entry[i].d_name);
			playerReturn = playMP3(fileToPlay, 1, testo, message);
			if (playerReturn == 2){
				break;
			}else if ((playerReturn == 1) | (playerReturn == 0)){
				//Controllo se sono in shuffle:
				if (playingMode == MODE_SHUFFLE){
					i = randomTrack(dirToPlay.number_of_directory_entries, playedTracks);
					if (i == -1){
						break;
					}
				}else{
					//Controllo se  finita la directory e non sono in repeat:
					if ((playerReturn == 0) & (i == dirToPlay.number_of_directory_entries - 1) & (playingMode != MODE_REPEAT)){
						break;
					}
					//Altrimenti passo al file successivo:
					i++;
					if (i > dirToPlay.number_of_directory_entries - 1){
						i = 0;
					}
				}
			}else if (playerReturn == -1){
				//Controllo se sono in shuffle:
				if (playingMode == MODE_SHUFFLE){
					i = randomTrack(dirToPlay.number_of_directory_entries, playedTracks);
					if (i == -1){
						break;
					}
				}else{
					i--;
					if (i < 0){
						i = dirToPlay.number_of_directory_entries - 1;
					}
				}
			}
		}
	}
	//Accendo il display se devo:	
	if (displayStatus == 0){
	  sceDisplayEnable();
	  //sceDisplaySetBrightness(curBrightness, 0);
	}
	opendir_close(&dirToPlay);
	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Play M3U:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void playM3U(char *m3uName){	
	char testo[10] = "";
	char fileToPlay[257] = "";
	int playerReturn = 0;
	int i = 0;
	struct M3U_songEntry song;
	char message[60] = "";

	M3U_open(m3uName);
	int songCount = M3U_getSongCount();
	char playedTracks[songCount + 1];

	char onlyName[257] = "";
	getFileName(m3uName, onlyName);
	snprintf(message, sizeof(message), "Playlist   : %-60.60s", onlyName);
	while (1){
		snprintf(testo, sizeof(testo), "%i/%i", i + 1, songCount);
		song = M3U_getSong(i);
		strcpy(fileToPlay, song.fileName);
		playerReturn = playMP3(fileToPlay, 1, testo, message);
		if (playerReturn == 2){
			break;
		}else if ((playerReturn == 1) | (playerReturn == 0)){
			//Controllo se sono in shuffle:
			if (playingMode == MODE_SHUFFLE){
				i = randomTrack(songCount, playedTracks);
				if (i == -1){
					break;
				}
			}else{
				//Controllo se  finita la playlist e non sono in repeat:
				if ((playerReturn == 0) & (i == songCount - 1) & (playingMode != MODE_REPEAT)){
					break;
				}
				//Altrimenti passo al file successivo:
				i++;
				if (i > songCount - 1){
					i = 0;
				}
			}
		}else if (playerReturn == -1){
			//Controllo se sono in shuffle:
			if (playingMode == MODE_SHUFFLE){
				i = randomTrack(songCount, playedTracks);
				if (i == -1){
					break;
				}
			}else{
				i--;
				if (i < 0){
					i = songCount - 1;
				}
			}
		}
	}
	//Accendo il display se devo:	
	if (displayStatus == 0){
	  sceDisplayEnable();
	  //sceDisplaySetBrightness(curBrightness, 0);
	}
	return;
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Main Menu:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void fileBrowser_menu()
	{
	struct opendir_struct directory;
	char curDir[257] = "";
	char dirToPlay[257] = "";
	char fileToPlay[257] = "";
	char ext[2][4] = {"MP3", "M3U"};
	char *result;

	if (strlen(fileBrowserDir) == 0){
		//Provo la prima directory:
		strcpy(curDir, mp3_directory_1);
		result = opendir_open(&directory, curDir, ext, 2, 1);
		if (result){
			//Provo la seconda directory:
			strcpy(curDir, mp3_directory_2);
			result = opendir_open(&directory, curDir, ext, 2, 1);
			if (result){
				screen_init();
				pspDebugScreenSetXY(0, 4);
				pspDebugScreenSetTextColor(RED);
				pspDebugScreenPrintf("\"%s\" and \"%s\" not found or empty", mp3_directory_1, mp3_directory_2);
				sceKernelDelayThread(2000000);
				pspDebugScreenSetXY(0, 5);
				pspDebugScreenPrintf("Quitting...");
				sceKernelDelayThread(1000000);
				current_mode = -1;
				return;
			}
		}
	}else{
		strcpy(curDir, fileBrowserDir);
		result = opendir_open(&directory, curDir, ext, 2, 1);
	}
	sortDirectory(directory);

	int selected_entry         = 0;
	int top_entry              = 0;
	int maximum_number_of_rows = 20;
	int starting_row           = 4;
	int updateInfo = 10;
	char author[50] = "";
	char title[100] = "";

	while (1)
		{
        SceCtrlData controller;
		screen_init();
		screen_menu_init();
		pspDebugScreenSetTextColor(WHITE);
		pspDebugScreenSetXY(0, 3);
		pspDebugScreenPrintf(curDir);

		while (1)
			{
			if (updateInfo >= 10)
			{
				screen_sysinfo();
				updateInfo = 0;
			}
			updateInfo++;

			sceCtrlReadBufferPositive(&controller, 1);
			if ((controller.Buttons & PSP_CTRL_DOWN) | (controller.Ly > 128 + ANALOG_SENS)){
				if (selected_entry + 1 < directory.number_of_directory_entries){
					selected_entry++;

					if (selected_entry == top_entry + maximum_number_of_rows){
						top_entry++;
						}
					}
			} else if ((controller.Buttons & PSP_CTRL_UP) | (controller.Ly < 128 - ANALOG_SENS)){
				if (selected_entry){
					selected_entry--;

					if (selected_entry == top_entry - 1){
						top_entry--;
					}
				}
			} else if ((controller.Buttons & PSP_CTRL_RIGHT) | (controller.Lx > 128 + ANALOG_SENS)){
				//Pagina gi:
				if (top_entry + maximum_number_of_rows < directory.number_of_directory_entries){
					top_entry = top_entry + maximum_number_of_rows;
					selected_entry += maximum_number_of_rows;
					if (selected_entry > directory.number_of_directory_entries - 1){
						selected_entry = directory.number_of_directory_entries - 1;
					}
				} else {
					selected_entry = directory.number_of_directory_entries - 1;
				}
				sceKernelDelayThread(100000);
			} else if ((controller.Buttons & PSP_CTRL_LEFT) | (controller.Lx < 128 - ANALOG_SENS)){
				//Pagina su:
				if (top_entry - maximum_number_of_rows >= 0){
					top_entry = top_entry - maximum_number_of_rows;
					selected_entry -= maximum_number_of_rows;
				} else {
					top_entry = 0;
					selected_entry = 0;
				}
				sceKernelDelayThread(100000);
			}
			pspDebugScreenSetXY(1, starting_row);
			pspDebugScreenSetTextColor(WHITE);
			pspDebugScreenSetBackColor(0xaa4400);

			//Segno precedente:
			if (top_entry > 0){
				pspDebugScreenPrintf("%-66.66s", "...");
			} else{
				pspDebugScreenPrintf("%-66.66s", "");
			}
			int i = 0;
			//Elementi della directory:
			for (; i < maximum_number_of_rows; i++){
				int current_entry = top_entry + i;

				pspDebugScreenSetXY(1, starting_row + i + 1);

				if (current_entry < directory.number_of_directory_entries){
					if (current_entry == selected_entry){
						pspDebugScreenSetBackColor(0x882200);
					} else {
						pspDebugScreenSetBackColor(0xcc6600);
					}
					//Controllo se  una directory o un file:
					if (FIO_S_ISREG(directory.directory_entry[current_entry].d_stat.st_mode)){
						if (strstr(directory.directory_entry[current_entry].d_name, ".m3u") != NULL || strstr(directory.directory_entry[current_entry].d_name, ".M3U") != NULL){
							pspDebugScreenSetTextColor(ORANGE);
						}else{
							pspDebugScreenSetTextColor(YELLOW);
						}
						pspDebugScreenPrintf("%-66.66s", directory.directory_entry[current_entry].d_name);
					} else if (FIO_S_ISDIR(directory.directory_entry[current_entry].d_stat.st_mode)){
						//Spezzo autore e titolo:
						if (splitAuthorTitle(directory.directory_entry[current_entry].d_name, author, title)){
							char format[10] = "";
							pspDebugScreenSetTextColor(YELLOW);
							pspDebugScreenPrintf("%s", author);
							pspDebugScreenPrintf(" ");
							pspDebugScreenSetTextColor(WHITE);
							snprintf(format, sizeof(format), "%%-%i.%is", 66 - strlen(author) - 1, 66 - strlen(author) - 1);
							pspDebugScreenPrintf(format, title);
						} else {
							pspDebugScreenSetTextColor(0xcccccc);
							pspDebugScreenPrintf("%-66.66s", directory.directory_entry[current_entry].d_name);
						}
					}
				} else {
					pspDebugScreenSetTextColor(WHITE);
					pspDebugScreenSetBackColor(0xaa4400);
					pspDebugScreenPrintf("%-66.66s", "");
				}
			}
			pspDebugScreenSetXY(1, starting_row + maximum_number_of_rows + 1);
			pspDebugScreenSetTextColor(WHITE);
			pspDebugScreenSetBackColor(0xaa4400);
			//Segno successivo:
			if (top_entry + maximum_number_of_rows < directory.number_of_directory_entries){
				pspDebugScreenPrintf("%-66.66s", "...");
			}else{
				pspDebugScreenPrintf("%-66.66s", "");
			}

			//Totale files:
			pspDebugScreenSetXY(55, starting_row + maximum_number_of_rows + 1);
			pspDebugScreenPrintf("Objects: %3.1i", directory.number_of_directory_entries);

			if (controller.Buttons & PSP_CTRL_CROSS || controller.Buttons & PSP_CTRL_CIRCLE || controller.Buttons & PSP_CTRL_TRIANGLE || 
				controller.Buttons & PSP_CTRL_SQUARE || controller.Buttons & PSP_CTRL_RTRIGGER || controller.Buttons & PSP_CTRL_LTRIGGER ||
				controller.Buttons & PSP_CTRL_START){
				break;
			}
			sceKernelDelayThread(100000);
		}

		if (controller.Buttons & PSP_CTRL_TRIANGLE){
			current_mode = -1;
			break;
		} else if (controller.Buttons & PSP_CTRL_CROSS) {
			//Controllo se  un file o una directory:
			if (FIO_S_ISREG(directory.directory_entry[selected_entry].d_stat.st_mode)){
				if (strstr(directory.directory_entry[selected_entry].d_name, ".mp3") != NULL || strstr(directory.directory_entry[selected_entry].d_name, ".MP3") != NULL){
					//File mp3:
					strcpy(fileToPlay, curDir);
					if (curDir[strlen(curDir)-1] != '/'){
						strcat(fileToPlay, "/");
					}
					strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
					playMP3(fileToPlay, 0, "", "");
					sceKernelDelayThread(200000);
				} else if (strstr(directory.directory_entry[selected_entry].d_name, ".m3u") != NULL || strstr(directory.directory_entry[selected_entry].d_name, ".M3U") != NULL){
					//File m3u:
					strcpy(fileToPlay, curDir);
					if (curDir[strlen(curDir)-1] != '/'){
						strcat(fileToPlay, "/");
					}
					strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
					playM3U(fileToPlay);
					sceKernelDelayThread(200000);
				}
			} else if (FIO_S_ISDIR(directory.directory_entry[selected_entry].d_stat.st_mode)){	
				if (curDir[strlen(curDir)-1] != '/'){
					strcat(curDir, "/");
				}
				strcat(curDir, directory.directory_entry[selected_entry].d_name);
				opendir_close(&directory);
				selected_entry = 0;
				top_entry = 0;
				result = opendir_open(&directory, curDir, ext, 2, 1);
				sortDirectory(directory);
				sceKernelDelayThread(200000);
			}
		} else if (controller.Buttons & PSP_CTRL_CIRCLE){
				if (directoryUp(curDir) == 0){
					opendir_close(&directory);
					selected_entry = 0;
					top_entry = 0;
					result = opendir_open(&directory, curDir, ext, 2, 1);
					sortDirectory(directory);
				}
				sceKernelDelayThread(200000);
		} else if (controller.Buttons & PSP_CTRL_SQUARE){
				//Controllo se  una directory:
				if (FIO_S_ISDIR(directory.directory_entry[selected_entry].d_stat.st_mode)){
					strcpy(dirToPlay, curDir);
					if (dirToPlay[strlen(dirToPlay)-1] != '/'){
						strcat(dirToPlay, "/");
					}
					strcat(dirToPlay, directory.directory_entry[selected_entry].d_name);
					playDirectory(dirToPlay);
					sceKernelDelayThread(200000);
				}
		} else if (controller.Buttons & PSP_CTRL_START){
			if (FIO_S_ISREG(directory.directory_entry[selected_entry].d_stat.st_mode)){
				//Aggiungo il file selezionato alla playlist:
				if (strstr(directory.directory_entry[selected_entry].d_name, ".mp3") != NULL || strstr(directory.directory_entry[selected_entry].d_name, ".MP3") != NULL){
					waitMessage("");
					//File mp3:
					strcpy(fileToPlay, curDir);
					if (curDir[strlen(curDir)-1] != '/'){
						strcat(fileToPlay, "/");
					}
					strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
					addFileToPlaylist(fileToPlay, 1);
				}
			}else if (FIO_S_ISDIR(directory.directory_entry[selected_entry].d_stat.st_mode)){
				//Aggiungo la directory alla playlist:
				waitMessage("000% done");
				strcpy(fileToPlay, curDir);
				if (curDir[strlen(curDir)-1] != '/'){
					strcat(fileToPlay, "/");
				}
				strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
				addDirectoryToPlaylist(fileToPlay);
			}
		} else if (controller.Buttons & PSP_CTRL_RTRIGGER){
			strcpy(fileBrowserDir, curDir);
			nextMode();
			break;
		} else if (controller.Buttons & PSP_CTRL_LTRIGGER){
			strcpy(fileBrowserDir, curDir);
			previousMode();
			break;
		}
	}
	opendir_close(&directory);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Playlist menu:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void playlist_menu(){
	struct opendir_struct directory;
	char curDir[257] = "";
	char lastOp[60] = "";
	char fileToPlay[257] = "";
	int selected_entry         = 0;
	int top_entry              = 0;
	int maximum_number_of_rows = 15;
	int starting_row           = 4;
	int updateInfo = 10;
	int sel_updateInfo = 0;
	int sel_songCount = 0;
	int sel_total = 0;

	screen_init();
	screen_playlist_init();
	strcpy(curDir, ebootDirectory);
	if (curDir[strlen(curDir)-1] != '/'){
		strcat(curDir, "/");
	}
	strcat(curDir, "playLists");
	char ext[1][4] = {"M3U"};
	char *result = opendir_open(&directory, curDir, ext, 1, 0);
	sortDirectory(directory);
	if (result){
		//current_mode = 0;
		//return;
	}else{
		//Carico i dati della prima playlist:
		strcpy(fileToPlay, curDir);
		if (fileToPlay[strlen(fileToPlay)-1] != '/'){
			strcat(fileToPlay, "/");
		}
		strcat(fileToPlay, directory.directory_entry[0].d_name);
		if (M3U_open(fileToPlay) == 0){
			sel_songCount = M3U_getSongCount();
			sel_total = M3U_getTotalLength();
			sel_updateInfo = 1;
		}
	}

	SceCtrlData controller;
	while (1){
		if (updateInfo >= 10){
			screen_sysinfo();
			updateInfo = 0;
		}
		updateInfo++;

		if (strlen(lastOp) > 0){
			pspDebugScreenSetTextColor(RED);
			pspDebugScreenSetBackColor(BLACK);
			pspDebugScreenSetXY(0, 3);
			pspDebugScreenPrintf(lastOp);
		}			
		sceCtrlReadBufferPositive(&controller, 1);
		if ((controller.Buttons & PSP_CTRL_DOWN) | (controller.Ly > 128 + ANALOG_SENS)){
			if (selected_entry + 1 < directory.number_of_directory_entries){
				selected_entry++;
				//Carico i dati della playlist selezionata:
				strcpy(fileToPlay, curDir);
				if (fileToPlay[strlen(fileToPlay)-1] != '/'){
					strcat(fileToPlay, "/");
				}
				strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
				if (M3U_open(fileToPlay) == 0){
					sel_songCount = M3U_getSongCount();
					sel_total = M3U_getTotalLength();
					sel_updateInfo = 1;
				}

				if (selected_entry == top_entry + maximum_number_of_rows){
					top_entry++;
					}
				}
		} else if ((controller.Buttons & PSP_CTRL_UP) | (controller.Ly < 128 - ANALOG_SENS)){
			if (selected_entry){
				selected_entry--;
				//Carico i dati della playlist selezionata:
				strcpy(fileToPlay, curDir);
				if (fileToPlay[strlen(fileToPlay)-1] != '/'){
					strcat(fileToPlay, "/");
				}
				strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
				if (M3U_open(fileToPlay) == 0){
					sel_songCount = M3U_getSongCount();
					sel_total = M3U_getTotalLength();
					sel_updateInfo = 1;
				}

				if (selected_entry == top_entry - 1){
					top_entry--;
				}
			}
		} else if ((controller.Buttons & PSP_CTRL_RIGHT) | (controller.Lx > 128 + ANALOG_SENS)){
			//Pagina gi:
			if (directory.number_of_directory_entries > 0){
				if (top_entry + maximum_number_of_rows < directory.number_of_directory_entries){
					top_entry = top_entry + maximum_number_of_rows;
					selected_entry += maximum_number_of_rows;
					if (selected_entry > directory.number_of_directory_entries - 1){
						selected_entry = directory.number_of_directory_entries - 1;
					}
				} else {
					selected_entry = directory.number_of_directory_entries - 1;
				}
				//Carico i dati della playlist selezionata:
				strcpy(fileToPlay, curDir);
				if (fileToPlay[strlen(fileToPlay)-1] != '/'){
					strcat(fileToPlay, "/");
				}
				strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
				if (M3U_open(fileToPlay) == 0){
					sel_songCount = M3U_getSongCount();
					sel_total = M3U_getTotalLength();
				}
				sceKernelDelayThread(100000);
			}
		} else if ((controller.Buttons & PSP_CTRL_LEFT) | (controller.Lx < 128 - ANALOG_SENS)){
			//Pagina su:
			if (directory.number_of_directory_entries > 0){
				if (top_entry - maximum_number_of_rows >= 0){
					top_entry = top_entry - maximum_number_of_rows;
					selected_entry -= maximum_number_of_rows;
				} else {
					top_entry = 0;
					selected_entry = 0;
				}
				//Carico i dati della playlist selezionata:
				strcpy(fileToPlay, curDir);
				if (fileToPlay[strlen(fileToPlay)-1] != '/'){
					strcat(fileToPlay, "/");
				}
				strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
				if (M3U_open(fileToPlay) == 0){
					sel_songCount = M3U_getSongCount();
					sel_total = M3U_getTotalLength();
					sel_updateInfo = 1;
				}
				sceKernelDelayThread(100000);
			}
		} else if (controller.Buttons & PSP_CTRL_CIRCLE && directory.number_of_directory_entries > 0) {
			if (confirm("Remove selected playlist?") == 1){
				//Remove selected playlist:
				strcpy(fileToPlay, curDir);
				if (fileToPlay[strlen(fileToPlay)-1] != '/'){
					strcat(fileToPlay, "/");
				}
				strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
				sceIoRemove(fileToPlay);
				//Ricarico il contenuto della directory
				opendir_close(&directory);
				selected_entry = 0;
				top_entry = 0;
				result = opendir_open(&directory, curDir, ext, 1, 0);
				sortDirectory(directory);
				if (directory.number_of_directory_entries > 0){
					//Carico i dati della playlist selezionata:
					strcpy(fileToPlay, curDir);
					if (fileToPlay[strlen(fileToPlay)-1] != '/'){
						strcat(fileToPlay, "/");
					}
					strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
					if (M3U_open(fileToPlay) == 0){
						sel_songCount = M3U_getSongCount();
						sel_total = M3U_getTotalLength();
						sel_updateInfo = 1;
					}
					sel_updateInfo = 1;
				}else{
					pspDebugScreenSetXY(0, 23);
					pspDebugScreenPrintf("%20.20s", "");
					pspDebugScreenSetXY(0, 24);
					pspDebugScreenPrintf("%20.20s", "");
				}
			}
		} else if (controller.Buttons & PSP_CTRL_SQUARE && directory.number_of_directory_entries > 0) {
			//Carico la playlist:
			if (FIO_S_ISREG(directory.directory_entry[selected_entry].d_stat.st_mode)){
				if (strstr(directory.directory_entry[selected_entry].d_name, ".m3u") != NULL || strstr(directory.directory_entry[selected_entry].d_name, ".M3U") != NULL){
					if (confirm("Load selected playlist?") == 1){
						strcpy(lastOp, "Error loading playlist");
						strcpy(fileToPlay, curDir);
						if (fileToPlay[strlen(fileToPlay)-1] != '/'){
							strcat(fileToPlay, "/");
						}
						strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
						if (M3U_open(fileToPlay) == 0){
							getFileName(fileToPlay, currentPlaylist);
							M3U_save(tempM3Ufile);
							playListModified = 0;
							strcpy(lastOp, "Playlist loaded");
						}
						sceKernelDelayThread(100000);
					}
				}
			}
		} else if (controller.Buttons & PSP_CTRL_CROSS && directory.number_of_directory_entries > 0) {
			//Controllo se  un file:
			if (FIO_S_ISREG(directory.directory_entry[selected_entry].d_stat.st_mode)){
				if (strstr(directory.directory_entry[selected_entry].d_name, ".m3u") != NULL || strstr(directory.directory_entry[selected_entry].d_name, ".M3U") != NULL){
					strcpy(fileToPlay, curDir);
					if (fileToPlay[strlen(fileToPlay)-1] != '/'){
						strcat(fileToPlay, "/");
					}
					strcat(fileToPlay, directory.directory_entry[selected_entry].d_name);
					playM3U(fileToPlay);
					screen_init();
					screen_playlist_init();
					sceKernelDelayThread(200000);
				}
			}
		}

		pspDebugScreenSetXY(1, starting_row);
		pspDebugScreenSetTextColor(WHITE);
		pspDebugScreenSetBackColor(0xaa4400);

		//Segno precedente:
		if (top_entry > 0){
			pspDebugScreenPrintf("%-66.66s", "...");
		} else{
			pspDebugScreenPrintf("%-66.66s", "");
		}
		int i = 0;
		for (; i < maximum_number_of_rows; i++){
			int current_entry = top_entry + i;

			pspDebugScreenSetXY(1, starting_row + i + 1);

			if (current_entry < directory.number_of_directory_entries){
				if (current_entry == selected_entry){
					pspDebugScreenSetBackColor(0x882200);
				} else {
					pspDebugScreenSetBackColor(0xcc6600);
				}
				//Controllo se  una directory o un file:
				if (FIO_S_ISREG(directory.directory_entry[current_entry].d_stat.st_mode)){
					if (strstr(directory.directory_entry[current_entry].d_name, ".m3u") != NULL || strstr(directory.directory_entry[current_entry].d_name, ".M3U") != NULL){
						pspDebugScreenSetTextColor(ORANGE);
					}else{
						pspDebugScreenSetTextColor(YELLOW);
					}
					pspDebugScreenPrintf("%-66.66s", directory.directory_entry[current_entry].d_name);
				} else {
					pspDebugScreenSetTextColor(0xcccccc);
					pspDebugScreenPrintf("%-66.66s", directory.directory_entry[current_entry].d_name);
				}
			} else {
				pspDebugScreenSetTextColor(WHITE);
				pspDebugScreenSetBackColor(0xaa4400);
				pspDebugScreenPrintf("%-66.66s", "");
			}
		}
		pspDebugScreenSetXY(1, starting_row + maximum_number_of_rows + 1);
		pspDebugScreenSetTextColor(WHITE);
		pspDebugScreenSetBackColor(0xaa4400);
		//Segno successivo:
		if (top_entry + maximum_number_of_rows < directory.number_of_directory_entries){
			pspDebugScreenPrintf("%-66.66s", "...");
		}else{
			pspDebugScreenPrintf("%-66.66s", "");
		}

		//Totale files:
		pspDebugScreenSetXY(55, starting_row + maximum_number_of_rows + 1);
		pspDebugScreenPrintf("Objects: %3.1i", directory.number_of_directory_entries);

		//Playlist info:
		if (sel_updateInfo == 1){
			pspDebugScreenSetTextColor(YELLOW);
			pspDebugScreenSetBackColor(BLACK);
			pspDebugScreenSetXY(0, 23);
			pspDebugScreenPrintf("Tracks    : %i", sel_songCount);
			pspDebugScreenPrintf("\n");
			int h = 0;
			int m = 0;
			int s = 0;
			h = sel_total / 3600;
			m = (sel_total - h * 3600) / 60;
			s = sel_total - h * 3600 - m * 60;
			pspDebugScreenPrintf("Total time: %2.2i:%2.2i:%2.2i", h, m, s);
			sel_updateInfo = 0;
		}

		//Tasti di uscita:
		if (controller.Buttons & PSP_CTRL_TRIANGLE || controller.Buttons & PSP_CTRL_RTRIGGER || controller.Buttons & PSP_CTRL_LTRIGGER){
			break;
		}
		sceKernelDelayThread(100000);
	}
	//Controllo input:
	if (controller.Buttons & PSP_CTRL_TRIANGLE){
		current_mode = -1;
	} else if (controller.Buttons & PSP_CTRL_RTRIGGER){
		nextMode();
	} else if (controller.Buttons & PSP_CTRL_LTRIGGER){
		previousMode();
	}
	opendir_close(&directory);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Playlist editor:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void playlist_editor(){
	char lastOp[60] = "";
	char M3Ufilename[257] = "";
	char newPlaylist[257] = "";
	int selected_entry         = 0;
	int top_entry              = 0;
	int maximum_number_of_rows = 14;
	int starting_row           = 4;
	int updateInfo = 10;
	struct ID3Tag ID3;
	struct M3U_songEntry song;
	int sel_updateInfo = 0;

	screen_init();
	//Apro il file temporaneo m3u:
	strcpy(M3Ufilename, tempM3Ufile);
	
	if (M3U_open(M3Ufilename) == 0){
		M3U_forceModified(playListModified);
		//Info primo file:
		if (M3U_getSongCount() > 0){
			song = M3U_getSong(0);
			ID3 = ParseID3(song.fileName);
			sel_updateInfo = 1;
		}

		SceCtrlData controller;
		screen_playlistEditor_init();
		pspDebugScreenSetTextColor(WHITE);
		pspDebugScreenSetXY(0, 20);
		if (strlen(currentPlaylist) > 0){
			pspDebugScreenPrintf("Filename: %-55.55s", currentPlaylist);
		}else{
			pspDebugScreenPrintf("Filename: %-55.55s", "NEW PLAYLIST");
		}

		while (1){
			if (updateInfo >= 10){
				screen_sysinfo();
				updateInfo = 0;
			}
			updateInfo++;

			if (strlen(lastOp) > 0){
				pspDebugScreenSetTextColor(RED);
				pspDebugScreenSetBackColor(BLACK);
				pspDebugScreenSetXY(0, 3);
				pspDebugScreenPrintf(lastOp);
			}			
			sceCtrlReadBufferPositive(&controller, 1);
			if ((controller.Buttons & PSP_CTRL_DOWN) | (controller.Ly > 128 + ANALOG_SENS)){
				if (selected_entry + 1 < M3U_getSongCount()){
					selected_entry++;
					song = M3U_getSong(selected_entry);
					ID3 = ParseID3(song.fileName);
					sel_updateInfo = 1;

					if (selected_entry == top_entry + maximum_number_of_rows){
						top_entry++;
					}
				}
			} else if ((controller.Buttons & PSP_CTRL_UP) | (controller.Ly < 128 - ANALOG_SENS)){
				if (selected_entry){
					selected_entry--;
					song = M3U_getSong(selected_entry);
					ID3 = ParseID3(song.fileName);
					sel_updateInfo = 1;

					if (selected_entry == top_entry - 1){
						top_entry--;
					}
				}
			} else if ((controller.Buttons & PSP_CTRL_RIGHT) | (controller.Lx > 128 + ANALOG_SENS)){
				//Pagina gi:
				if (M3U_getSongCount() > 0){
					if (top_entry + maximum_number_of_rows < M3U_getSongCount()){
						top_entry = top_entry + maximum_number_of_rows;
						selected_entry += maximum_number_of_rows;
						if (selected_entry > M3U_getSongCount() - 1){
							selected_entry = M3U_getSongCount() - 1;
						}
					} else {
						selected_entry = M3U_getSongCount() - 1;
					}
					song = M3U_getSong(selected_entry);
					ID3 = ParseID3(song.fileName);
					sel_updateInfo = 1;
					sceKernelDelayThread(100000);
				}
			} else if ((controller.Buttons & PSP_CTRL_LEFT) | (controller.Lx < 128 - ANALOG_SENS)){
				//Pagina su:
				if (M3U_getSongCount() > 0){
					if (top_entry - maximum_number_of_rows >= 0){
						top_entry = top_entry - maximum_number_of_rows;
						selected_entry -= maximum_number_of_rows;
					} else {
						top_entry = 0;
						selected_entry = 0;
					}
					song = M3U_getSong(selected_entry);
					ID3 = ParseID3(song.fileName);
					sel_updateInfo = 1;
					sceKernelDelayThread(100000);
				}
			} else if (controller.Buttons & PSP_CTRL_CROSS && M3U_getSongCount() > 0){
				//Sposto gi:
				if (M3U_moveSongDown(selected_entry) == 0){
					selected_entry++;
					if (selected_entry == top_entry + maximum_number_of_rows){
						top_entry++;
					}
					sceKernelDelayThread(100000);
				}
			} else if (controller.Buttons & PSP_CTRL_SQUARE && M3U_getSongCount() > 0){
				//Sposto su:
				if (M3U_moveSongUp(selected_entry) == 0){
					selected_entry--;
					if (selected_entry == top_entry - 1){
						top_entry--;
					}
					sceKernelDelayThread(100000);
				}
			} else if (controller.Buttons & PSP_CTRL_CIRCLE && M3U_getSongCount() > 0){
				if (confirm("Remove selected track from playlist?") == 1){
					M3U_removeSong(selected_entry);
					sceKernelDelayThread(100000);
					if (selected_entry >= M3U_getSongCount()){
						selected_entry--;
					}
					song = M3U_getSong(selected_entry);
					ID3 = ParseID3(song.fileName);
					sel_updateInfo = 1;
					if (selected_entry == top_entry - 1){
						top_entry--;
					}
				}
			} else if (controller.Buttons & PSP_CTRL_START && M3U_getSongCount() > 0){
				if (confirm("Save the playlist?") == 1){
					if (askString(currentPlaylist) == 0){
						//Controllo l'estensione:
						if (strstr(currentPlaylist, ".m3u") == NULL && strstr(currentPlaylist, ".M3U") == NULL){
							strcat(currentPlaylist, ".m3u");
						}
						//Salvo la playlist:
						strcpy(newPlaylist, playlistDir);
						strcat(newPlaylist, "/");
						strcat(newPlaylist, currentPlaylist);
						M3U_save(newPlaylist);
						strcpy(lastOp, "Palylist saved");
						pspDebugScreenSetXY(0, 20);
						pspDebugScreenSetBackColor(BLACK);
						pspDebugScreenSetTextColor(WHITE);
						pspDebugScreenPrintf("Filename: %-55.55s", currentPlaylist);
						pspDebugScreenSetXY(0, 21);
						pspDebugScreenPrintf("%-20.20s", "");
					}
				}
			} else if (controller.Buttons & PSP_CTRL_SELECT && M3U_getSongCount() > 0){
				if (confirm("Clear the playlist?") == 1){
					M3U_clear();
					strcpy(currentPlaylist, "");
					pspDebugScreenSetXY(0, 20);
					pspDebugScreenSetBackColor(BLACK);
					pspDebugScreenSetTextColor(WHITE);
					pspDebugScreenPrintf("Filename: %-55.55s", "NEW PLAYLIST");
					sceKernelDelayThread(100000);
				}
			} else if (controller.Buttons & PSP_CTRL_NOTE && M3U_getSongCount() > 0){
				if (confirm("Play the playlist?") == 1){
					M3U_save(M3Ufilename);
					playM3U(M3Ufilename);
					screen_init();
					screen_playlistEditor_init();
					sel_updateInfo = 1;
					sceKernelDelayThread(200000);
				}
			}
			pspDebugScreenSetXY(1, starting_row);
			pspDebugScreenSetTextColor(WHITE);
			pspDebugScreenSetBackColor(0xaa4400);

			//Segno precedente:
			if (top_entry > 0){
				pspDebugScreenPrintf("%-66.66s", "...");
			}else{
				pspDebugScreenPrintf("%-66.66s", "");
			}
			int i = 0;
			for (; i < maximum_number_of_rows; i++){
				int current_entry = top_entry + i;

				pspDebugScreenSetXY(1, starting_row + i + 1);

				if (current_entry < M3U_getSongCount()){
					if (current_entry == selected_entry){
						pspDebugScreenSetBackColor(0x882200);
					} else {
						pspDebugScreenSetBackColor(0xcc6600);
					}
					pspDebugScreenSetTextColor(YELLOW);
					pspDebugScreenPrintf("%-66.66s", M3U_getSong(current_entry).title);
				} else {
					pspDebugScreenSetTextColor(WHITE);
					pspDebugScreenSetBackColor(0xaa4400);
					pspDebugScreenPrintf("%-66.66s", "");
				}
			}
			pspDebugScreenSetXY(1, starting_row + maximum_number_of_rows + 1);
			pspDebugScreenSetTextColor(WHITE);
			pspDebugScreenSetBackColor(0xaa4400);
			//Segno successivo:
			if (top_entry + maximum_number_of_rows < M3U_getSongCount()){
				pspDebugScreenPrintf("%-66.66s", "...");
			}else{
				pspDebugScreenPrintf("%-66.66s", "");
			}

			//Totale tracce:
			pspDebugScreenSetXY(56, starting_row + maximum_number_of_rows + 1);
			pspDebugScreenPrintf("Tracks: %3.1i", M3U_getSongCount());

			//Informazioni sulla playlist:
			if (M3U_isModified() == 1 && strlen(currentPlaylist) > 0){
				pspDebugScreenSetTextColor(RED);
				pspDebugScreenSetBackColor(BLACK);
				pspDebugScreenSetXY(0, 21);
				pspDebugScreenPrintf("Unsaved changes");
			}

			//Informazioni sulla traccia:
			if (sel_updateInfo == 1){
				pspDebugScreenSetTextColor(YELLOW);
				pspDebugScreenSetBackColor(BLACK);
				pspDebugScreenSetXY(0, 22);
				pspDebugScreenPrintf("Artist   : %-30.30s", ID3.ID3Artist);
				pspDebugScreenPrintf("\n");
				pspDebugScreenPrintf("Title    : %-30.30s", ID3.ID3Title);
				pspDebugScreenPrintf("\n");
				pspDebugScreenPrintf("Album    : %-30.30s", ID3.ID3Album);
				sel_updateInfo = 0;
			}
			
			//Tasti di uscita:
			if (controller.Buttons & PSP_CTRL_TRIANGLE || controller.Buttons & PSP_CTRL_RTRIGGER || controller.Buttons & PSP_CTRL_LTRIGGER){
				break;
			}
			sceKernelDelayThread(100000);
		}
		//Salvo lo stato di modifica della playlist:
		playListModified = M3U_isModified();

		//Controllo input:
		if (controller.Buttons & PSP_CTRL_TRIANGLE){
			current_mode = -1;
		} else if (controller.Buttons & PSP_CTRL_RTRIGGER){
			M3U_save(M3Ufilename);
			nextMode();
		} else if (controller.Buttons & PSP_CTRL_LTRIGGER){
			M3U_save(M3Ufilename);
			previousMode();
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Main
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int main() {
	scePowerSetClockFrequency(222, 222, 111);
	scePowerSetCpuClockFrequency(80);
	scePowerSetBusClockFrequency(60);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
	SetupCallbacks();
	tzset(); 

	//openLog("ms0:/lightMP3.log");
	//Controllo directory corrente e creo directory per playlist:
	getcwd(ebootDirectory, 256);
	strcpy(playlistDir, ebootDirectory);
	if (playlistDir[strlen(playlistDir)-1] != '/'){
		strcat(playlistDir, "/");
	}
	strcat(playlistDir, "playLists");
	sceIoMkdir(playlistDir, 0777);

	//Nome per playlist temporanea:
	strcpy(tempM3Ufile, ebootDirectory);
	if (tempM3Ufile[strlen(tempM3Ufile)-1] != '/'){
		strcat(tempM3Ufile, "/");
	}
	strcat(tempM3Ufile, "temp.m3u");
	FILE *f;
	f = fopen(tempM3Ufile, "w");
	fclose(f);

	//Controllo luminosit:
	//checkBrightness();

	//Equalizer init:
	EQ_init();

	srand(time(NULL));
	pspAudioInit();
	current_mode = 0;
	while (current_mode >= 0){
		if (current_mode == 0){
			fileBrowser_menu();
		}else if (current_mode == 1){
			playlist_menu();
		}else if (current_mode == 2){
			playlist_editor();
		}
	}

	pspAudioEnd();
	sceKernelExitGame();
	return(0);
}
